@@ -71,6 +71,11 @@ class HealSparseMap(object):
             # this is a healpix_map input
             if sentinel is None:
                 sentinel = hp.UNSEEN
+            if is_integer_value(healpix_map[0]) and not is_integer_value(sentinel):
+                raise ValueError("The sentinel must be set to an integer value with an integer healpix_map")
+            elif not is_integer_value(healpix_map[0]) and is_integer_value(sentinel):
+                raise ValueError("The sentinel must be set to an float value with an float healpix_map")
+
             self._cov_map, self._sparse_map = self.convert_healpix_map(healpix_map,
                                                                        nside_coverage=nside_coverage,
                                                                        nest=nest,
@@ -454,7 +459,7 @@ class HealSparseMap(object):
 
         return cov_map, sparse_map
 
-    def write(self, filename, clobber=False):
+    def write(self, filename, clobber=False, nocompress=False):
         """
         Write heal HealSparseMap to filename.  Use the `metadata` property from
         the map to persist additional information in the fits header.
@@ -465,6 +470,9 @@ class HealSparseMap(object):
            Name of file to save
         clobber : `bool`, optional
            Clobber existing file?  Default is False.
+        nocompress : `bool`, optional
+           If this is False, then integer maps will be compressed losslessly.
+           Note that `np.int64` maps cannot be compressed in the FITS standard.
         """
         if os.path.isfile(filename) and not clobber:
             raise RuntimeError("Filename %s exists and clobber is False." % (filename))
@@ -483,8 +491,19 @@ class HealSparseMap(object):
         if self._is_wide_mask:
             s_hdr['WIDEMASK'] = self._is_wide_mask
             s_hdr['WWIDTH'] = self._wide_mask_width
-            _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map.ravel())
+            # Wide masks can be compressed.
+            _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map.ravel(),
+                            compress=not nocompress,
+                            compress_tilesize=self._wide_mask_width*self._cov_map.nfine_per_cov)
+        elif ((self.is_integer_map and self._sparse_map[0].dtype.itemsize < 8) or
+              (not self.is_integer_map and not self._is_rec_array)):
+            # Integer maps < 64 bit (8 byte) can be compressed, as can
+            # floating point maps
+            _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map,
+                            compress=not nocompress,
+                            compress_tilesize=self._cov_map.nfine_per_cov)
         else:
+            # All other maps are not compressed.
             _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map)
 
     def _reserve_cov_pix(self, new_cov_pix):
@@ -542,7 +561,7 @@ class HealSparseMap(object):
                 # Special for wide_mask
                 if not isinstance(values, np.ndarray):
                     raise ValueError("Wide mask must be set with a numpy ndarray")
-                if len(values) == self._wide_mask_width:
+                if len(values) == self._wide_mask_width and len(values.shape) == 1:
                     is_single_value = True
                     # Reshape so we can use the 0th entry below
                     _values = _values.reshape((1, self._wide_mask_width))
@@ -584,7 +603,7 @@ class HealSparseMap(object):
 
         if self._is_view:
             # Check that we are not setting new pixels
-            if np.any(self.get_values_pix(_pix) <= self._sentinel):
+            if np.any(self.get_values_pix(_pix) == self._sentinel):
                 raise RuntimeError("This API cannot be used to set new pixels in the map.")
 
         # Compute the coverage pixels
@@ -695,7 +714,7 @@ class HealSparseMap(object):
 
         self.update_values_pix(pixels, value, nest=nest, operation='and')
 
-    def get_values_pos(self, theta_or_ra, phi_or_dec, lonlat=False, valid_mask=False):
+    def get_values_pos(self, theta_or_ra, phi_or_dec, lonlat=True, valid_mask=False):
         """
         Get the map value for the position.  Positions may be theta/phi
         co-latitude and longitude in radians, or longitude and latitude in
@@ -754,16 +773,16 @@ class HealSparseMap(object):
 
         if valid_mask:
             if self._is_rec_array:
-                return (values[self._primary] > self._sentinel)
+                return (values[self._primary] != self._sentinel)
             elif self._is_wide_mask:
                 return (values > 0).sum(axis=1, dtype=np.bool)
             else:
-                return (values > self._sentinel)
+                return (values != self._sentinel)
         else:
             # Just return the values
             return values
 
-    def check_bits_pos(self, theta_or_ra, phi_or_dec, bits, lonlat=False):
+    def check_bits_pos(self, theta_or_ra, phi_or_dec, bits, lonlat=True):
         """
         Check the bits at the map for an array of positions.  Positions may be
         theta/phi co-latitude and longitude in radians, or longitude and
@@ -841,15 +860,27 @@ class HealSparseMap(object):
            Float array of fractional coverage of each pixel
         """
 
-        cov_map = np.zeros_like(self.coverage_mask, dtype=np.double)
+        cov_map = np.zeros_like(self.coverage_mask, dtype=np.float64)
         cov_mask = self.coverage_mask
         npop_pix = np.count_nonzero(cov_mask)
-        if self._is_rec_array:
-            spMap_T = self._sparse_map[self._primary].reshape((npop_pix + 1, -1))
+        if self._is_wide_mask:
+            shape_new = (npop_pix + 1,
+                         self._cov_map.nfine_per_cov,
+                         self._wide_mask_width)
+            sp_map_t = self._sparse_map.reshape(shape_new)
+            # This trickery first checks all the bits, and then sums into the
+            # coverage pixel
+            counts = np.sum(np.any(sp_map_t != self._sentinel, axis=2), axis=1)
         else:
-            spMap_T = self._sparse_map.reshape((npop_pix + 1, -1))
-        counts = np.sum((spMap_T > self._sentinel), axis=1).astype(np.double)
-        cov_map[cov_mask] = counts[1:] / 2**self._cov_map.bit_shift
+            shape_new = (npop_pix + 1,
+                         self._cov_map.nfine_per_cov)
+            if self._is_rec_array:
+                sp_map_t = self._sparse_map[self._primary].reshape(shape_new)
+            else:
+                sp_map_t = self._sparse_map.reshape(shape_new)
+            counts = np.sum((sp_map_t != self._sentinel), axis=1).astype(np.float64)
+
+        cov_map[cov_mask] = counts[1:]/self._cov_map.nfine_per_cov
         return cov_map
 
     @property
@@ -864,6 +895,63 @@ class HealSparseMap(object):
         """
         return self._cov_map.coverage_mask
 
+    def fracdet_map(self, nside):
+        """
+        Get the fractional area covered by the sparse map at an arbitrary resolution.
+
+        Parameters
+        ----------
+        nside : `int`
+           Healpix nside for fracdet map.  Must not be greater than sparse
+           resolution or less than coverage resolution.
+
+        Returns
+        -------
+        fracdet_map : `HealSparseMap`
+           Fractional coverage map.
+        """
+        if nside > self.nside_sparse:
+            raise ValueError("Cannot return fracdet_map at higher resolution than "
+                             "the sparse map (nside=%d)." % (self.nside_sparse))
+        if nside < self.nside_coverage:
+            raise ValueError("Cannot return fractdet_map at lower resolution than "
+                             "the coverage map (nside=%d)." % (self.nside_coverage))
+
+        # This code is essentially a unification of coverage_map() and degrade()
+        # to get the fracdet_coverage in a single step
+        cov_mask = self.coverage_mask
+        npop_pix = np.count_nonzero(cov_mask)
+
+        bit_shift = _compute_bitshift(nside, self.nside_sparse)
+        nfine_per_frac = 2**bit_shift
+        nfrac_per_cov = self._cov_map.nfine_per_cov//nfine_per_frac
+
+        if self._is_wide_mask:
+            shape_new = ((npop_pix + 1)*nfrac_per_cov,
+                         nfine_per_frac,
+                         self._wide_mask_width)
+            sp_map_t = self._sparse_map.reshape(shape_new)
+            fracdet = np.sum(np.any(sp_map_t != self._sentinel, axis=2), axis=1).astype(np.float64)
+        else:
+            shape_new = ((npop_pix + 1)*nfrac_per_cov,
+                         nfine_per_frac)
+            if self._is_rec_array:
+                sp_map_t = self._sparse_map[self._primary].reshape(shape_new)
+            else:
+                sp_map_t = self._sparse_map.reshape(shape_new)
+            fracdet = np.sum(sp_map_t != self._sentinel, axis=1).astype(np.float64)
+
+        fracdet /= nfine_per_frac
+
+        fracdet_cov_map = HealSparseCoverage.make_from_pixels(self.nside_coverage,
+                                                              nside,
+                                                              np.where(cov_mask)[0])
+
+        # The sentinel for a fracdet_map is 0.0, no coverage.
+        return HealSparseMap(cov_map=fracdet_cov_map, sparse_map=fracdet,
+                             nside_sparse=nside, primary=self._primary,
+                             sentinel=0.0)
+
     @property
     def nside_coverage(self):
         """
@@ -1018,7 +1106,7 @@ class HealSparseMap(object):
 
             self._metadata = metadata
 
-    def generate_healpix_map(self, nside=None, reduction='mean', key=None):
+    def generate_healpix_map(self, nside=None, reduction='mean', key=None, nest=True):
         """
         Generate the associated healpix map
 
@@ -1037,6 +1125,8 @@ class HealSparseMap(object):
         key : `str`
             If the parent HealSparseMap contains recarrays, key selects the
             field that will be transformed into a HEALPix map.
+        nest : `bool`, optional
+            Output healpix map should be in nest format?
 
         Returns
         -------
@@ -1077,7 +1167,9 @@ class HealSparseMap(object):
         hp_map = np.full(hp.nside2npix(nside), hp.UNSEEN, dtype=dtypeOut)
 
         valid_pixels = single_map.valid_pixels
-        hp_map[valid_pixels] = single_map.get_values_pix(valid_pixels)
+        if not nest:
+            valid_pixels = hp.nest2ring(nside, valid_pixels)
+        hp_map[valid_pixels] = single_map.get_values_pix(valid_pixels, nest=nest)
 
         return hp_map
 
@@ -1091,15 +1183,15 @@ class HealSparseMap(object):
         valid_pixels : `np.ndarray`
         """
         if self._is_rec_array:
-            valid_pixel_inds, = np.where(self._sparse_map[self._primary] > self._sentinel)
+            valid_pixel_inds, = np.where(self._sparse_map[self._primary] != self._sentinel)
         elif self._is_wide_mask:
-            valid_pixel_inds, = np.where(self._sparse_map.sum(axis=1, dtype=np.bool))
+            valid_pixel_inds, = np.where(np.any(self._sparse_map != self._sentinel, axis=1))
         else:
-            valid_pixel_inds, = np.where(self._sparse_map > self._sentinel)
+            valid_pixel_inds, = np.where(self._sparse_map != self._sentinel)
 
         return valid_pixel_inds - self._cov_map[self._cov_map.cov_pixels_from_index(valid_pixel_inds)]
 
-    def valid_pixels_pos(self, lonlat=False, return_pixels=False):
+    def valid_pixels_pos(self, lonlat=True, return_pixels=False):
         """
         Get an array with the position of valid pixels in the sparse map.
 
@@ -1146,16 +1238,27 @@ class HealSparseMap(object):
         """
         if self._nside_sparse < nside_out:
             raise ValueError('nside_out should be smaller than nside for the sparse_map')
-        if self._is_wide_mask:
-            raise NotImplementedError('Cannot degrade a wide_mask map')
         # Count the number of filled pixels in the coverage mask
         npop_pix = np.count_nonzero(self.coverage_mask)
         # We need the new bit_shifts and we have to build a new CovIndexMap
         bit_shift = _compute_bitshift(self.nside_coverage, nside_out)
         nfine_per_cov = 2**bit_shift
+
+        # Work with wide masks
+        if self._is_wide_mask:
+            if reduction not in ['and', 'or']:
+                raise NotImplementedError('Cannot degrade a wide_mask map with this \
+                reduction operation, try and/or')
+            else:
+                nbits = self._sparse_map.shape[1]
+                aux = self._sparse_map.reshape((npop_pix+1, (nside_out//self.nside_coverage)**2, nbits, -1))
+                new_sparse_map = reduce_array(aux, reduction=reduction, axis=3).reshape((-1, nbits))
+                sentinel_out = self._sentinel
+
         # Work with RecArray (we have to change the resolution to all maps...)
-        if self._is_rec_array:
+        elif self._is_rec_array:
             dtype = []
+            sentinel_out = hp.UNSEEN
             # We should avoid integers
             for key, value in self._sparse_map.dtype.fields.items():
                 if issubclass(self._sparse_map[key].dtype.type, np.integer):
@@ -1170,33 +1273,35 @@ class HealSparseMap(object):
                 aux = aux.reshape((npop_pix + 1, (nside_out//self.nside_coverage)**2, -1))
                 # Perform the reduction operation (check utils.reduce_array)
                 aux = reduce_array(aux, reduction=reduction)
-                # Transform back to UNSEEN
-                aux[np.isnan(aux)] = hp.UNSEEN
+                # Transform back to sentinel value
+                aux[np.isnan(aux)] = sentinel_out
                 new_sparse_map[key] = aux
 
-        # Work with regular ndarray
+        # Work with int array and ndarray
+        elif (issubclass(self._sparse_map.dtype.type, np.integer)) and (reduction in ['and', 'or']):
+            aux = self._sparse_map.reshape((npop_pix+1, (nside_out//self.nside_coverage)**2, -1))
+            new_sparse_map = reduce_array(aux, reduction=reduction)
+            sentinel_out = self._sentinel
         else:
             if issubclass(self._sparse_map.dtype.type, np.integer):
                 aux_dtype = np.float64
             else:
                 aux_dtype = self._sparse_map.dtype
-
+            sentinel_out = hp.UNSEEN
             aux = self._sparse_map.astype(aux_dtype)
             aux[self._sparse_map == self._sentinel] = np.nan
             aux = aux.reshape((npop_pix + 1, (nside_out//self.nside_coverage)**2, -1))
             aux = reduce_array(aux, reduction=reduction)
             # NaN are converted to UNSEEN
-            aux[np.isnan(aux)] = hp.UNSEEN
+            aux[np.isnan(aux)] = sentinel_out
             new_sparse_map = aux
 
         # The coverage index map is now offset, we have to build a new one
         new_cov_map = HealSparseCoverage.make_from_pixels(self.nside_coverage,
                                                           nside_out,
                                                           np.where(self.coverage_mask)[0])
-
         return HealSparseMap(cov_map=new_cov_map, sparse_map=new_sparse_map,
-                             nside_coverage=self.nside_coverage,
-                             nside_sparse=nside_out, primary=self._primary, sentinel=hp.UNSEEN)
+                             nside_sparse=nside_out, primary=self._primary, sentinel=sentinel_out)
 
     def apply_mask(self, mask_map, mask_bits=None, mask_bit_arr=None, in_place=True):
         """
@@ -1372,12 +1477,44 @@ class HealSparseMap(object):
 
         new_sparse_map = np.full_like(self._sparse_map[key], _sentinel)
 
-        valid_indices = (self._sparse_map[self._primary] > self._sentinel)
+        valid_indices = (self._sparse_map[self._primary] != self._sentinel)
         new_sparse_map[valid_indices] = self._sparse_map[key][valid_indices]
 
         return HealSparseMap(cov_map=self._cov_map, sparse_map=new_sparse_map,
                              nside_sparse=self._nside_sparse, sentinel=_sentinel)
 
+    def astype(self, dtype, sentinel=None):
+        """
+        Convert sparse map to a different numpy datatype, including sentinel
+        values.  If sentinel is not specified the default for the converted
+        datatype is used (`healpy.UNSEEN` for float, and -MAXINT for ints).
+
+        Parameters
+        ----------
+        dtype : `numpy.dtype`
+            Valid numpy dtype for a single array.
+        sentinel : `int` or `float`, optional
+            Converted map sentinel value.
+
+        Returns
+        -------
+        sparse_map : `HealSparseMap`
+            New map with new data type.
+        """
+        if self._is_rec_array:
+            raise RuntimeError("Cannot convert datatype of a recarray map.")
+        elif self._is_wide_mask:
+            raise RuntimeError("Cannot convert datatype of a wide mask.")
+
+        new_sparse_map = self._sparse_map.astype(dtype)
+        _sentinel = check_sentinel(new_sparse_map.dtype.type, sentinel)
+
+        invalid_pix = (self._sparse_map == self._sentinel)
+        new_sparse_map[invalid_pix] = _sentinel
+
+        return HealSparseMap(cov_map=self._cov_map, sparse_map=new_sparse_map,
+                             nside_sparse=self.nside_sparse, sentinel=_sentinel)
+
     def __add__(self, other):
         """
         Add a constant.
@@ -1587,14 +1724,14 @@ class HealSparseMap(object):
                 raise NotImplementedError("Can only use an integer constant with the %s operation" % (name))
 
         if self._is_wide_mask:
-            valid_sparse_pixels = (self._sparse_map > self._sentinel).sum(axis=1, dtype=np.bool)
+            valid_sparse_pixels = (self._sparse_map != self._sentinel).sum(axis=1, dtype=np.bool)
 
             other_value = np.zeros(self._wide_mask_width, self._sparse_map.dtype)
             for bit in other:
                 field, bitval = _get_field_and_bitval(bit)
                 other_value[field] |= bitval
         else:
-            valid_sparse_pixels = (self._sparse_map > self._sentinel)
+            valid_sparse_pixels = (self._sparse_map != self._sentinel)
 
         if in_place:
             if self._is_wide_mask:
